{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
@dataclass
{%- if base_class %}
class {{ class_name }}({{ base_class }}):
{%- else %}
class {{ class_name }}:
{%- endif %}
{%- if description %}
    """
    {{ description }}
    """
{%- endif %}
{%- if not fields %}
    pass
{%- endif %}
{%- for field in fields -%}
    {%- if field|length > 2 %}
    {{ field[0] }}: {{ field[1] }} = {{field[2]}}
    {%- else %}
    {{ field[0] }}: {{ field[1] }}
    {%- endif %}
{%- endfor -%}
