{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
class {{ class_name }}(Enum):
{%- if description %}
    """
    {{ description }}
    """
{%- endif %}
{%- for field in fields %}
    {{ field[0] }} = {{ field[1] }}
{%- endfor -%}
