{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
class {{ class_name }}({{ base_model }}):
{%- if description %}
    """
    auto gen by FastApi quick CRUD
    """
{%- endif %}
{%- if config %}
{%- filter indent(4) %}
{% include 'Config.jinja2' %}
{%- endfilter %}
{%- endif %}
{%- if not field %}
    pass
{%- else %}
    total: int
    result: List[{{ field[0] }}]
{%- endif %}

    class Config:
        orm_mode = True
